/*
  J. Neira
  J. A. Castellanos
  Robotics and Real Time Group
  University of Zaragoza
  
  sp_matrix.h
  Implements basic MATRIX operations
*/

#ifndef _SP_MATRIX_H
#define _SP_MATRIX_H

#define MAX_ROWS (7)
#define MAX_COLS (7)

typedef struct {
  int   rows;
  int   cols;
  float data[MAX_ROWS][MAX_COLS];
} MATRIX;

typedef struct {
  int   elements;
  float data[MAX_ROWS];
} VECTOR;

#define DOF (3)

typedef struct {
  int mat[DOF];
  int range;
} BMAT;

#define MROWS(m)             ((m).rows)
#define MCOLS(m)             ((m).cols)
#define MDATA(m,i,j)         ((m).data[i][j])

#define VELEMENTS(v)         ((v).elements)
#define VDATA(v,i)           ((v).data[i])

#define M_SQUARE(m)          ((m).rows == (m).cols)
#define M_COMPAT_DIM(m, n)   ((m).cols == (n).rows)
#define M_EQUAL_DIM(m, n)    (((m).rows == (n).rows)  && ((m).cols == (n).cols))
#define V_EQUAL_DIM(v, w)    (((v).elements == (w).elements))
#define MV_COMPAT_DIM(m, v)  ((m).cols == (v).elements)

#define FIRST(b)             ((b).mat[0])
#define SECOND(b)            ((b).mat[1])
#define THIRD(b)             ((b).mat[2])
#define RANGE(b)             ((b).range)

#define SQUARE(x)            ((x)*(x))

MATRIX create_matrix (int rows, int cols);
void initialize_matrix (MATRIX *m, int rows, int cols);
MATRIX CHEPE_diagonal_matrix (int dim, float el1, float el2, float el3);
void diagonal_matrix (MATRIX *m, int dim, float el1, float el2, float el3);
void print_matrix (char *message, MATRIX const *m);
void identity_matrix (MATRIX *m, int d);
void transpose_matrix (MATRIX const *m, MATRIX *r);
void add_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r);
void substract_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r);
void multiply_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r);
void pp_multiply_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r);
void multiply_matrix_scalar (float s, MATRIX const *m, MATRIX *r);
void negate_matrix (MATRIX const *m, MATRIX *r);
VECTOR create_vector (int elements);
void initialize_vector (VECTOR *v, int elements);
void print_vector (char *message, VECTOR const *v);
void add_vector (VECTOR const *v, VECTOR const *w, VECTOR *r);
void substract_vector (VECTOR const *v, VECTOR const *w, VECTOR *r);
void multiply_scalar_vector (float s, VECTOR const *v, VECTOR *r);
void negate_vector (VECTOR const *v, VECTOR *r);
float cross_product (MATRIX const *m, int f1, int c1, int f2, int c2);
float determinant (MATRIX const *m);
void inverse_matrix (MATRIX const *m, MATRIX *n);
void multiply_matrix_vector (MATRIX const *m, VECTOR const *v, VECTOR *r);
void pp_multiply_t_vector_matrix (VECTOR const *v, MATRIX const *m,float *d);
BMAT bmat (int first, int second, int third);
void initialize_bmat (BMAT *b, int first, int second, int third);
void print_bmat (char *message, BMAT const *b);
void multiply_bmat_vector (BMAT const *b, VECTOR const *v, VECTOR *r);
void multiply_t_bmat_vector (BMAT const *b, VECTOR const *v, VECTOR *r);
void copy_row (MATRIX const *s, MATRIX *d, int srow, int drow, int cols);
void copy_col (MATRIX const *s, MATRIX *d, int scol, int dcol, int rows);
void pp_multiply_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r);
void fill_row (MATRIX *d, int row, int cols);
void fill_col (MATRIX *d, int col, int rows);
void pp_multiply_t_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r);
void multiply_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r);
void multiply_matrix_t_bmat (MATRIX const *m, BMAT const *b, MATRIX *r);
void multiply_matrix_bmat (MATRIX const *m, BMAT const *b, MATRIX *r);

/* New functions added 1996-07 */
void multiply_matrix_t_matrix(MATRIX const *a, MATRIX const *b, MATRIX *r);
void multiply_matrix_matrix_t_matrix(MATRIX const *a, MATRIX const *b, MATRIX const *c, MATRIX *r);

#endif

