---------------------------------------------------------------------
-- Fichero:   simulador_teclado.adb
-- Autor:     J.L.Villarroel, C.Sagues, J.D.Tardos
-- Version:   v2.0 4-4-2002
-- Proposito: Generar pseudo-interrupciones de teclado
---------------------------------------------------------------------
-- v2.0  4-4-2002  JDT
--    Se crea este con cosas de Eventos Aperiodicos y mensaje
---------------------------------------------------------------------
with MaRTE_OS;
with Ada.Real_Time, System; use Ada.Real_Time;

package body Simulador_Teclado is

   dato_teclado : Character := '0';
   pragma Atomic (dato_teclado);

   type Parameterless_Handler is access protected procedure;
   User_Handler : Parameterless_Handler;

   N : constant := 12;
   type contador is mod N;
   el_mensaje   : String (1 .. N) := "TIEMPO REAL ";
   Separaciones : array (contador) of Time_Span :=
     (8 | 9 | 10 | 11=> Milliseconds (2500),
      3 | 6 | 7    => Milliseconds (1000),
      others => Milliseconds (250));

   task Simulador is
      pragma Priority (system.Priority'Last);
      entry comienzo;
   end Simulador;

   task body Simulador is
      i         : contador := 0;
      siguiente : Time;
   begin
      accept comienzo;
      siguiente := Clock;
      loop
         siguiente := siguiente + Separaciones (i);
         delay until siguiente;
         dato_teclado := el_mensaje (Integer (i) + 1);
         i            := i + 1;
         User_Handler.all;
      end loop;
   end Simulador;

   procedure Attach_Handler (New_Handler : access protected procedure) is
   -- Conecta las interrupciones de teclado simuladas con el
   -- manejador New_Handler e inicia la simulacion del teclado.
   -- Cada vez que se simule una pulsacion de tecla, se llamara
   -- al manejador. La tecla habra que leerla con leer_tecla
   begin
      User_Handler := New_Handler;
      Simulador.comienzo;
   end Attach_Handler;

   procedure Detach_Handler is
   -- Desconecta el manejador y termina la simulacion de teclado
   begin
      abort Simulador;
   end Detach_Handler;

   function leer_caracter return Character is
   -- Lee el ultimo caracter recibido.
   -- Si no se lee a tiempo, se pierde
   begin
      return dato_teclado;
   end leer_caracter;

end Simulador_Teclado;
