#ifndef __YENTA_H
#define __YENTA_H

// #include <asm/io.h>
#include <sys/pci.h> // For pci functions (pci_find_device...)
#include <sys/io.h> // For readb, writeb
#include <intr.h> // For interrupt handler
#include <semaphore.h> // For synchronize threads and interrupt handler
#include "marte_linux_yenta_layer.h" // For linux fake functions and constants
#include <unistd.h> // for sleep

#define CB_SOCKET_EVENT		0x00
#define    CB_CSTSEVENT		0x00000001	/* Card status event */
#define    CB_CD1EVENT		0x00000002	/* Card detect 1 change event */
#define    CB_CD2EVENT		0x00000004	/* Card detect 2 change event */
#define    CB_PWREVENT		0x00000008	/* PWRCYCLE change event */

#define CB_SOCKET_MASK		0x04
#define    CB_CSTSMASK		0x00000001	/* Card status mask */
#define    CB_CDMASK		0x00000006	/* Card detect 1&2 mask */
#define    CB_PWRMASK		0x00000008	/* PWRCYCLE change mask */

#define CB_SOCKET_STATE		0x08
#define    CB_CARDSTS		0x00000001	/* CSTSCHG status */
#define    CB_CDETECT1		0x00000002	/* Card detect status 1 */
#define    CB_CDETECT2		0x00000004	/* Card detect status 2 */
#define    CB_PWRCYCLE		0x00000008	/* Socket powered */
#define    CB_16BITCARD		0x00000010	/* 16-bit card detected */
#define    CB_CBCARD		0x00000020	/* CardBus card detected */
#define    CB_IREQCINT		0x00000040	/* READY(xIRQ)/xCINT high */
#define    CB_NOTACARD		0x00000080	/* Unrecognizable PC card detected */
#define    CB_DATALOST		0x00000100	/* Potential data loss due to card removal */
#define    CB_BADVCCREQ		0x00000200	/* Invalid Vcc request by host software */
#define    CB_5VCARD		0x00000400	/* Card Vcc at 5.0 volts? */
#define    CB_3VCARD		0x00000800	/* Card Vcc at 3.3 volts? */
#define    CB_XVCARD		0x00001000	/* Card Vcc at X.X volts? */
#define    CB_YVCARD		0x00002000	/* Card Vcc at Y.Y volts? */
#define    CB_5VSOCKET		0x10000000	/* Socket Vcc at 5.0 volts? */
#define    CB_3VSOCKET		0x20000000	/* Socket Vcc at 3.3 volts? */
#define    CB_XVSOCKET		0x40000000	/* Socket Vcc at X.X volts? */
#define    CB_YVSOCKET		0x80000000	/* Socket Vcc at Y.Y volts? */

#define CB_SOCKET_FORCE		0x0C
#define    CB_FCARDSTS		0x00000001	/* Force CSTSCHG */
#define    CB_FCDETECT1		0x00000002	/* Force CD1EVENT */
#define    CB_FCDETECT2		0x00000004	/* Force CD2EVENT */
#define    CB_FPWRCYCLE		0x00000008	/* Force PWREVENT */
#define    CB_F16BITCARD	0x00000010	/* Force 16-bit PCMCIA card */
#define    CB_FCBCARD		0x00000020	/* Force CardBus line */
#define    CB_FNOTACARD		0x00000080	/* Force NOTACARD */
#define    CB_FDATALOST		0x00000100	/* Force data lost */
#define    CB_FBADVCCREQ	0x00000200	/* Force bad Vcc request */
#define    CB_F5VCARD		0x00000400	/* Force 5.0 volt card */
#define    CB_F3VCARD		0x00000800	/* Force 3.3 volt card */
#define    CB_FXVCARD		0x00001000	/* Force X.X volt card */
#define    CB_FYVCARD		0x00002000	/* Force Y.Y volt card */
#define    CB_CVSTEST		0x00004000	/* Card VS test */

#define CB_SOCKET_CONTROL	0x10
#define  CB_SC_VPP_MASK		0x00000007
#define   CB_SC_VPP_OFF		0x00000000
#define   CB_SC_VPP_12V		0x00000001
#define   CB_SC_VPP_5V		0x00000002
#define   CB_SC_VPP_3V		0x00000003
#define   CB_SC_VPP_XV		0x00000004
#define   CB_SC_VPP_YV		0x00000005
#define  CB_SC_VCC_MASK		0x00000070
#define   CB_SC_VCC_OFF		0x00000000
#define   CB_SC_VCC_5V		0x00000020
#define   CB_SC_VCC_3V		0x00000030
#define   CB_SC_VCC_XV		0x00000040
#define   CB_SC_VCC_YV		0x00000050
#define  CB_SC_CCLK_STOP	0x00000080

#define CB_SOCKET_POWER		0x20
#define    CB_SKTACCES		0x02000000	/* A PC card access has occurred (clear on read) */
#define    CB_SKTMODE		0x01000000	/* Clock frequency has changed (clear on read) */
#define    CB_CLKCTRLEN		0x00010000	/* Clock control enabled (RW) */
#define    CB_CLKCTRL		0x00000001	/* Stop(0) or slow(1) CB clock (RW) */

/*
 * Cardbus configuration space
 */
#define CB_BRIDGE_BASE(m)	(0x1c + 8*(m))
#define CB_BRIDGE_LIMIT(m)	(0x20 + 8*(m))
#define CB_BRIDGE_CONTROL	0x3e
#define   CB_BRIDGE_CPERREN	0x00000001
#define   CB_BRIDGE_CSERREN	0x00000002
#define   CB_BRIDGE_ISAEN	0x00000004
#define   CB_BRIDGE_VGAEN	0x00000008
#define   CB_BRIDGE_MABTMODE	0x00000020
#define   CB_BRIDGE_CRST	0x00000040
#define   CB_BRIDGE_INTR	0x00000080
#define   CB_BRIDGE_PREFETCH0	0x00000100
#define   CB_BRIDGE_PREFETCH1	0x00000200
#define   CB_BRIDGE_POSTEN	0x00000400
#define CB_LEGACY_MODE_BASE	0x44

/*
 * ExCA area extensions in Yenta
 */
#define CB_MEM_PAGE(map)	(0x40 + (map))


/* control how 16bit cards are powered */
#define YENTA_16BIT_POWER_EXCA	0x00000001
#define YENTA_16BIT_POWER_DF	0x00000002


struct yenta_socket;

struct cardbus_type {
	int	(*override)(struct yenta_socket *);
	void	(*save_state)(struct yenta_socket *);
	void	(*restore_state)(struct yenta_socket *);
	int	(*sock_init)(struct yenta_socket *);
};

struct pcmcia_socket {
//     struct module           *owner;
//     spinlock_t          lock;
//     socket_state_t          socket;
//     u_int               state;
//     u_short             functions;
//     u_short             lock_count;
//     pccard_mem_map          cis_mem;
//     void __iomem            *cis_virt;
//     struct {
//         u_int           AssignedIRQ;
//         u_int           Config;
//     } irq;
//     io_window_t         io[MAX_IO_WIN];
//     window_t            win[MAX_WIN];
//     struct list_head        cis_cache;
//     u_int               fake_cis_len;
//     char                *fake_cis;
//     struct list_head        socket_list;
//     struct completion       socket_released;
//     /* deprecated */
//     unsigned int            sock;       /* socket number */
//     /* socket capabilities */
//     u_int               features;
    unsigned int               irq_mask;
//     u_int               map_size;
//     kio_addr_t          io_offset;
//     u_char              pci_irq;
//     struct pci_dev *        cb_dev;
//     /* socket setup is done so resources should be able to be allocated. Only
//     * if set to 1, calls to find_{io,mem}_region are handled, and insertion
//     * events are actually managed by the PCMCIA layer.*/
//     u8              resource_setup_done:1;
//     /* is set to one if resource setup is done using adjust_resource_info() */
//     u8              resource_setup_old:1;
//     u8              resource_setup_new:1;
//
//     u8              reserved:5;
//     /* socket operations */
//     struct pccard_operations *  ops;
//     struct pccard_resource_ops *    resource_ops;
//     void *              resource_data;
//     /* Zoom video behaviour is so chip specific its not worth adding
//     this to _ops */
//     void                (*zoom_video)(struct pcmcia_socket *, int);
//     /* so is power hook */
//     int (*power_hook)(struct pcmcia_socket *sock, int operation);
//     /* allows tuning the CB bridge before loading driver for the CB card */
//     void (*tune_bridge)(struct pcmcia_socket *sock, struct pci_bus *bus);
//     /* state thread */
//     struct mutex            skt_mutex;  /* protects socket h/w state */
//     struct task_struct      *thread;
//     struct completion       thread_done;
//     wait_queue_head_t       thread_wait;
//     spinlock_t          thread_lock;    /* protects thread_events */
//     unsigned int            thread_events;
//     /* pcmcia (16-bit) */
//     struct pcmcia_callback      *callback;
//
//     /* cardbus (32-bit) */
//     struct resource *       cb_cis_res;
//     void __iomem            *cb_cis_virt;
//     /* socket device */
//     struct device           dev;
//     void                *driver_data;   /* data internal to the socket driver */
};

struct yenta_socket {
	struct pci_device *dev;
	int cb_irq, io_irq;
	void __iomem *base;
// 	struct timer_list poll_timer;

	struct pcmcia_socket socket;
	struct cardbus_type *type;

	u32 flags;

	/* for PCI interrupt probing */
	unsigned int probe_status;

	/* A few words of private data for special stuff of overrides... */
	unsigned int private[8];

	/* PCI saved state */
	u32 saved_state[2];
};

extern int __devinit yenta_probe (struct pci_device *dev, const struct pci_device_id *id);

typedef struct socket_state_t {
  unsigned int  flags;
  unsigned int       csc_mask;
  unsigned char      Vcc, Vpp;
  unsigned char      io_irq;
} socket_state_t;

/* Events */
#define CS_EVENT_PRI_LOW        0
#define CS_EVENT_PRI_HIGH       1

#define CS_EVENT_WRITE_PROTECT      0x000001
#define CS_EVENT_CARD_LOCK      0x000002
#define CS_EVENT_CARD_INSERTION     0x000004
#define CS_EVENT_CARD_REMOVAL       0x000008
#define CS_EVENT_BATTERY_DEAD       0x000010
#define CS_EVENT_BATTERY_LOW        0x000020
#define CS_EVENT_READY_CHANGE       0x000040
#define CS_EVENT_CARD_DETECT        0x000080
#define CS_EVENT_RESET_REQUEST      0x000100
#define CS_EVENT_RESET_PHYSICAL     0x000200
#define CS_EVENT_CARD_RESET     0x000400
#define CS_EVENT_REGISTRATION_COMPLETE  0x000800
#define CS_EVENT_PM_SUSPEND     0x002000
#define CS_EVENT_PM_RESUME      0x004000
#define CS_EVENT_INSERTION_REQUEST  0x008000
#define CS_EVENT_EJECTION_REQUEST   0x010000
#define CS_EVENT_MTD_REQUEST        0x020000
#define CS_EVENT_ERASE_COMPLETE     0x040000
#define CS_EVENT_REQUEST_ATTENTION  0x080000
#define CS_EVENT_CB_DETECT      0x100000
#define CS_EVENT_3VCARD         0x200000
#define CS_EVENT_XVCARD         0x400000

/* Return codes */
#define CS_SUCCESS      0x00
#define CS_BAD_ADAPTER      0x01
#define CS_BAD_ATTRIBUTE    0x02
#define CS_BAD_BASE     0x03
#define CS_BAD_EDC      0x04
#define CS_BAD_IRQ      0x06
#define CS_BAD_OFFSET       0x07
#define CS_BAD_PAGE     0x08
#define CS_READ_FAILURE     0x09
#define CS_BAD_SIZE     0x0a
#define CS_BAD_SOCKET       0x0b
#define CS_BAD_TYPE     0x0d
#define CS_BAD_VCC      0x0e
#define CS_BAD_VPP      0x0f
#define CS_BAD_WINDOW       0x11
#define CS_WRITE_FAILURE    0x12
#define CS_NO_CARD      0x14
#define CS_UNSUPPORTED_FUNCTION 0x15
#define CS_UNSUPPORTED_MODE 0x16
#define CS_BAD_SPEED        0x17
#define CS_BUSY         0x18
#define CS_GENERAL_FAILURE  0x19
#define CS_WRITE_PROTECTED  0x1a
#define CS_BAD_ARG_LENGTH   0x1b
#define CS_BAD_ARGS     0x1c
#define CS_CONFIGURATION_LOCKED 0x1d
#define CS_IN_USE       0x1e
#define CS_NO_MORE_ITEMS    0x1f
#define CS_OUT_OF_RESOURCE  0x20
#define CS_BAD_HANDLE       0x21

#endif

