/*******************************************************
 NAME:		KFPT.c
 AUTHOR:	Luis Montesano
 DATE:		07-04-04
 DESCRIPTION:
	This is the source file for the KFPT functions that 
	implement the tracking of moving objects using Kalman
	Filters. It provides an interface based on the previous
	estimations and the incoming data. It uses the KF_equations
	and the listaKFPT modules.

********************************************************/

#include <stdio.h>
#include "KFPT.h"
#include "imported/calcul.h"
#include "math.h"
#include <stdlib.h>
#include "imported/basicOp.h"


// CONSTANTS
#define DIST_TH1 1.0
#define DIST_TH2 1.0
#define DIST_TH3 0.5*0.5
#define MIN_NB_PTOS 4
#define maxNum 361
// DATA TYPES



// VARIABLES
static Tscan unknown_k1;
static Tsc Srobot_k1;
static float cosAng[NUMPUNTOSLASER];
static float sinAng[NUMPUNTOSLASER];
static short filterLabel;
static short numFilters;

// 


static void createNewFilter(TnodoKFPT **KFS, Tpf position);


// FUNCTIONS
/**************************************************
  Initializes the internal variables 

 initKFPT()

 INPUT:
   
 OUTPUT:
   
***************************************************/
void initKFPT() {
	int i;

	filterLabel=0;
	numFilters=0;
	unknown_k1.numPuntos=0;

	for (i=0;i<NUMPUNTOSLASER;i++){
		cosAng[i]=(float)cos(PI/360*i-PI/2.0); /* Los cos y sen se pueden precomp */
		fprintf(stderr,"%f\t",cosAng[i]);
		sinAng[i]=(float)sin(PI/360*i-PI/2.0);
	}
	fprintf(stderr,"\n");

}

/**************************************************
  Prediction step of the tracked moving objects

 void predictFilers(TnodoKFPT *KFS, float period){	

 INPUT:
	KFS
	period
   
 OUTPUT:
	KFS
   
***************************************************/
void predictFilers(TnodoKFPT *KFS, float period){	
  
  TnodoKFPT *sig;
//   	fprintf(stderr,"predictFilers\n");

  sig=KFS;
  while (sig!=NULL) {

//   	fprintf(stderr,"predictFilers_i");
	  predKFPT(&(sig->filter), period, &(sig->filter));
	  sig=(TnodoKFPT*)sig->next;
// 	fprintf(stderr," acaba\n");
  }

}

/**************************************************
  Update step of the tracked moving objects and Data
  Association between the filters and the detected 
  moving laser scan points

 void updateFilters(TnodoKFPT *KFS, Tscan const *moving){

 INPUT:
	KFS
	moving: TScan containing the dynamic points
   
 OUTPUT:
	KFS
   
***************************************************/


//Paso las variables a globales porque sino MaRTE en ejecucion
// falla (MaRTE Assertion failure)
//   short filterInd[NUMPUNTOSLASER];
//   float minsqdist[NUMPUNTOSLASER];

  short pointInd[maxNum][NUMPUNTOSLASER];
//   short numPointInd[maxNum];
  
//   Tpf z;
//   Tscan moving_w;
//   Tpi segments[maxNum];


void updateFilters(TnodoKFPT **KFS, Tscan const *moving, Tsc *Srobot){
  
  TnodoKFPT *sig, *parent;
  Tpf inPf, filters[maxNum];
  short numFilter,I, J;  
  
  short filterInd[NUMPUNTOSLASER];
  float minsqdist[NUMPUNTOSLASER];
// 
//   short pointInd[maxNum][NUMPUNTOSLASER];
  short numPointInd[maxNum];
//   
  Tpf z;
  Tscan moving_w;
  Tpi segments[maxNum];
  short nsegments;
  float DIST_TH=1.5;
  //float DIST_TH3=1.5;
  float sqdist;
  short nb_ptos;
  short min_nb_ptos=4;

//  fprintf(stderr,"UPDATE FILTERS\n");

  // Data Association ------------------------------------------
  // -----------------------------------------------------------
// fprintf(stderr,"Data Association\n");
  sig=*KFS;
  numFilter=0;
  while (sig!=NULL){
// fprintf(stderr,"Data Association_sig\n");

	  inPf.x=sig->filter.x.data[0];
	  inPf.y=sig->filter.x.data[1];
	  filters[numFilter]=inPf;	  
	  numPointInd[numFilter]=0;
	  numFilter=numFilter+1;
	  sig=(TnodoKFPT*)sig->next;
  }
// fprintf(stderr,"BUCLE_1\n");

  numPointInd[maxNum-1]=0;
  for (I=0;I<moving->numPuntos; I++) {
// fprintf(stderr,"BUCLE_1_ind_%d\n",I);
	  transfor_directa_p(moving->laserC[I].x, moving->laserC[I].y, Srobot, &(moving_w.laserC[I]));	  
	  
	  filterInd[I]=maxNum-1;
	  minsqdist[I]=DIST_TH;
	  for (J=0; J<numFilter; J++) {		  				
		  sqdist=(filters[J].x -moving_w.laserC[I].x)*(filters[J].x -moving_w.laserC[I].x) +
				 (filters[J].y -moving_w.laserC[I].y)*(filters[J].y -moving_w.laserC[I].y);
		  if (sqdist<minsqdist[I]){
			  filterInd[I]=J;
			  minsqdist[I]=sqdist;	
		  }
	  }	  

	 
	  pointInd[filterInd[I]][numPointInd[filterInd[I]]]=I;
	  numPointInd[filterInd[I]]++;	  
  }  

  
//  fprintf(stderr,"Update existing filters\n");
  // Update existing filters ---------------------------------------
  // ---------------------------------------------------------------
  sig=*KFS; I=0;
  while (sig!=NULL) {
// fprintf(stderr,"Update existing filters_i\n");
	  if (numPointInd[I]==0) {
// fprintf(stderr,"Update existing filters_i_1\n");
		  sig->numNotSeen++;		  
	  }

	  else {
// fprintf(stderr,"Update existing filters_i_1\n");
		  z.x=0; z.y=0;
		  for (J=0; J<numPointInd[I]; J++) {
// 			fprintf(stderr,"I=%d,J=%d\n",I,J);
			  z.x=z.x+moving_w.laserC[pointInd[I][J]].x;
			  z.y=z.y+moving_w.laserC[pointInd[I][J]].y;
		  }
		  z.x=z.x/numPointInd[I];z.y=z.y/numPointInd[I];
// fprintf(stderr,"Update existing filters_i_1\n");
		  updateKFPT(&(sig->filter), &z, &(sig->filter));
// fprintf(stderr,"Update existing filters_i_1\n");
		  sig->numNotSeen=0;
// fprintf(stderr,"Update existing filters_i_1\n");
	  }
	  I++;
// fprintf(stderr,"Update existing filters_i_1\n");
	  sig=(TnodoKFPT*)sig->next;
// fprintf(stderr,"Update existing filters_i_1\n");
  }
// fprintf(stderr,"Remove filters without support\n");

  // Remove filters without support
  sig=*KFS; parent=NULL;
  while (sig!=NULL) {
//  fprintf(stderr,"Remove filters without support_i\n");
	  if (sig->numNotSeen>3){
		  if (parent==NULL) {
			  *KFS=(TnodoKFPT*)sig->next;
			  free(sig);
			  sig=*KFS;
		  }
		  else {			  
			  parent->next=sig->next;		  
			  free(sig);
			  sig=(TnodoKFPT*)parent->next;
		  }		  
		  numFilters--;
	  }	  
	  else {
		  parent=sig;
		  sig=(TnodoKFPT*)sig->next;
	  }
  }
// fprintf(stderr,"Create New Clusters ... \n");

  // Create New Clusters    
  nsegments=1;
  segments[nsegments].x=0;
    
  for (I=1; I<numPointInd[maxNum-1]; I++) {
// fprintf(stderr,"Create New Clusters_%d\n",I);
// fprintf(stderr,"Create New ClustersAux_1\n");
	  if ((moving->laserC[pointInd[maxNum-1][I]].x-moving->laserC[pointInd[maxNum-1][I-1]].x) * 
		  (moving->laserC[pointInd[maxNum-1][I]].x-moving->laserC[pointInd[maxNum-1][I-1]].x) +	  
		  (moving->laserC[pointInd[maxNum-1][I]].y-moving->laserC[pointInd[maxNum-1][I-1]].y) * 
		  (moving->laserC[pointInd[maxNum-1][I]].y-moving->laserC[pointInd[maxNum-1][I-1]].y)>DIST_TH3) 
	  {          
// fprintf(stderr,"Create New ClustersAux_2\n");
		  nb_ptos=I-segments[nsegments].x;
		  if (nb_ptos>min_nb_ptos) {
             segments[nsegments].y=I-1;
// fprintf(stderr,"Create New ClustersAux_3\n");

			 z.x=0; z.y=0;
             for (J=segments[nsegments].x; J<=segments[nsegments].y; J++) {
// fprintf(stderr,"J=%d\n",J);

				 z.x=z.x+moving_w.laserC[pointInd[maxNum-1][J]].x;
				 z.y=z.y+moving_w.laserC[pointInd[maxNum-1][J]].y;
			 }
			 z.x=z.x/nb_ptos;z.y=z.y/nb_ptos;

			 createNewFilter(KFS, z);			 
             nsegments=nsegments+1;			 
          }
// fprintf(stderr,"Create New ClustersAux_4\n");

          segments[nsegments].x=I;
	  }
  }
  
  nb_ptos=I-segments[nsegments].x;
  if (nb_ptos>min_nb_ptos) {       
	  segments[nsegments].y=I-1;
	  
	  z.x=0; z.y=0;
	  for (J=segments[nsegments].x; J<=segments[nsegments].y; J++) {
		  z.x=z.x+moving_w.laserC[pointInd[maxNum-1][J]].x;
		  z.y=z.y+moving_w.laserC[pointInd[maxNum-1][J]].y;
	  }
	  z.x=z.x/nb_ptos;z.y=z.y/nb_ptos;
	  createNewFilter(KFS, z);	  
  }
  else {
	  nsegments=nsegments-1;
  }

}


/**************************************************
 Creates a new filter

 void createNewFilter(TnodoKFPT *KFS, Tpf position){

 INPUT:
	KFS	
   
 OUTPUT:
	KFS
   
***************************************************/

void createNewFilter(TnodoKFPT **KFS, Tpf position) {
	Tkf newKF;

	numFilters++;
	filterLabel++;
	initializeKF(&position, &newKF);
	InsertarListaKFPT(KFS, &newKF, filterLabel, filterLabel);
	
}



short IsStaticProb_L1(short i, short j, float mapa[][DIMMATRIZ]) {
 if (mapa[i][j]>0) 
    return 1;
 else 
   return 0;
}
 
short IsStaticProb_L2(short i, short j, float mapa[][DIMMATRIZ]) {

 if (i<DIMMATRIZ-1 && mapa[i+1][j]>0)
    return 1;
       
 if (j<DIMMATRIZ-1 && mapa[i][j+1]>0)
    return 1;
 
 if (i>0 && mapa[i-1][j]>0)
    return 1;
 
 if (j>0 && mapa[i][j-1]>0)
    return 1;
 
 if (i<DIMMATRIZ-1 && j<DIMMATRIZ-1 && mapa[i+1][j+1]>0)
    return 1;
 
 if (i<DIMMATRIZ-1 && j>0 && mapa[i+1][j-1]>0)
    return 1;
 
 if (i>0 && j<DIMMATRIZ-1 && mapa[i-1][j+1]>0)
    return 1;
 
 if (i>0 && j>0 && mapa[i-1][j-1]>0)
    return 1;

 return 0;
}

short IsBeingTracked(short i, short j, Tpcelda *filters, short numFilters, short *filter) {

  int diff, minDiff;
  short I;

  *filter=-1;
  minDiff=2;

  // No se utiliza la covarianza de los filtros para calcular el emparejamientos
  for (I=0; I<numFilters; I++) {
  	  diff=(filters[I].fil - i)*(filters[I].fil - i) + (filters[I].col - j)*(filters[I].col - j);
	  if (diff<minDiff) {
	  	  *filter=I;
	  }
  }

  if (*filter==-1) 
	  return 0;
  else return 1;

}

short IsMovingProb(short i, short j, float mapa[][DIMMATRIZ]) {
  short iinf, isup, jinf, jsup;
  short I,J,cnt1,cnt2,res;
    
  if (i>0) iinf=i-1; else iinf=0;
  if (i<DIMMATRIZ-1) isup=i+1; else isup=DIMMATRIZ-1;
  if (j>0) jinf=j-1; else jinf=0;
  if (j<DIMMATRIZ-1) jsup=j+1; else jsup=DIMMATRIZ-1;
  
  cnt1=0; cnt2=0;
  for (I=iinf; I<=isup; I++)
	  for (J=jinf; J<=jsup; J++) 
		  if (mapa[I][J]<-0.5)
			  cnt1++;		  
		  else cnt2++;
	  

  if (cnt1+cnt2==9) 
	  res= mapa[i][j]<0 && cnt1>7;
  else res=mapa[i][j]<0;

  return res;
}


/**************************************************
 Points of laser are clasified into static, dynamic or unknown according to the map 
 and the tracked objects

void clasificarPuntos(float *laser, float **mapa, TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi)

 INPUT:
	laser
	mapa
	KFS	
	Srobot
	Smap
   
 OUTPUT:
	classi
   
***************************************************/
void clasificarPuntos(float *laser, float mapa[][DIMMATRIZ], TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi) {

	//fprintf(stderr,"CLASIFICAR PUNTOS\n");

/*	for (int i=0;i<NUMPUNTOSLASER;i++){
		fprintf(stderr,"%f\t",cosAng[i]);
	}
		fprintf(stderr,"\n");
*/



	int i,j;
	static int discontinuidades[NUMPUNTOSLASER];
	int numDisc;
	static Tpcelda scanCells[NUMPUNTOSLASER];	  
	static Tpcelda unknown_k1Cells[NUMPUNTOSLASER];
	Tsc PosMat,tmp2,tmp3;Tpf inPf;
	float DISC_TH=0.35;
	int outCnt;
	
	// Compute the matrix coordinates of the previous unknown points  
	inversion_sis(SMap, &tmp2);  
	composicion_sis(&tmp2,&Srobot_k1,&PosMat);
	for (i=0; i<unknown_k1.numPuntos; i++) {
		transfor_directa_p (unknown_k1.laserC[i].x,unknown_k1.laserC[i].y,
			&PosMat,&inPf);
		SisM2celda(&inPf, &unknown_k1Cells[i]);
	}


	inversion_sis(SMap, &tmp2);
	composicion_sis(&tmp2,SRobot,&tmp3);
	composicion_sis(&tmp3,&PosSensorRobot,&PosMat);
	outCnt=0;
	for (i=0; i<NUMPUNTOSLASER; i++) {
		if (laser[i] <MAXRANGOLASER){
		    transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosSensorRobot,&classi->robotScan.laserC[i]);
			transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosMat,&inPf);			
			SisM2celda(&inPf,&scanCells[i]);	
			SisM2celda(&inPf,&classi->scanCells[i]);
			if (mapa[scanCells[i].fil][scanCells[i].col]>0.5)
				classi->tag[i]=STATIC;
			else classi->tag[i]=UNKNOWN;
		}
		else {
			transfor_directa_p (MAXRANGOLASER*cosAng[i],	MAXRANGOLASER*sinAng[i],
				&PosSensorRobot,&classi->outPtR.laserC[outCnt]);
			classi->robotScan.laserC[i]=classi->outPtR.laserC[outCnt];
			classi->tag[i]=OUT_OF_RANGE;
			outCnt++;
		}
	}
//	for (i=0; i<NUMPUNTOSLASER; i++) {
//	fprintf(stderr,"%d\ttag=%d\n",i,classi->tag[i]);  
//	}
	classi->robotScan.numPuntos=NUMPUNTOSLASER;
	classi->outPtR.numPuntos=outCnt;

	numDisc=1; discontinuidades[0]=0;
	for (i=1; i<NUMPUNTOSLASER; i++) {		
		if (fabs(laser[i]-laser[i-1])>DISC_TH) {
			discontinuidades[numDisc]=i;
			numDisc++;
		}		
	}
	discontinuidades[numDisc]=NUMPUNTOSLASER;
	numDisc++;
// 	fprintf(stderr,"Numero de discontinuidades\t%d\n",numDisc);

	for (i=1; i<numDisc-1; i++) {
		// Buscar derecha
		for (j=discontinuidades[i]-1; j>=discontinuidades[i-1]; j--){
//			fprintf(stderr,"1_tag_j=%d\n",classi->tag[j]);
//			fprintf(stderr,"mapa[scanCells[j].fil][scanCells[j].col]=%f\n",mapa[scanCells[j].fil][scanCells[j].col]);

			if (classi->tag[j]==OUT_OF_RANGE || mapa[scanCells[j].fil][scanCells[j].col]>-0.5){
				break;
			}
			else{
				classi->tag[j]=MOVING;
			}
		}
		// Buscar Izquierda
		for (j=discontinuidades[i]; j<discontinuidades[i+1]; j++){
//			fprintf(stderr,"2_tag_j=%d\n",classi->tag[j]);
//			fprintf(stderr,"mapa[scanCells[j].fil][scanCells[j].col]=%f\n",mapa[scanCells[j].fil][scanCells[j].col]);
			if (classi->tag[j]==OUT_OF_RANGE || mapa[scanCells[j].fil][scanCells[j].col]>-0.5)
				break;
			else{
				classi->tag[j]=MOVING;
			}
		}
	}
	
	for (i=0; i<NUMPUNTOSLASER; i++){
		if (classi->tag[i]==3){
			for (j=0; j<unknown_k1.numPuntos; j++) {
				if (unknown_k1Cells[j].col==scanCells[i].col && 
					unknown_k1Cells[j].fil==scanCells[i].fil) {
					classi->tag[i]=STATIC;
					break;
				}
			}			
		}
	}
//	for (i=0; i<NUMPUNTOSLASER; i++) {
//	fprintf(stderr,"%d\ttag=%d\n",i,classi->tag[i]);  
//	}


  classi->staticPtR.numPuntos=0;
  classi->movingPtR.numPuntos=0;
  classi->unknownPtR.numPuntos=0;



  for (i=0; i<NUMPUNTOSLASER; i++){
//	  classi->tag[i]=1;
	  switch(classi->tag[i]){
	  case 1: classi->staticPtR.laserC[classi->staticPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->staticPtR.numPuntos++;
		  break;
	  case 2: classi->movingPtR.laserC[classi->movingPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->movingPtR.numPuntos++;
		  break;
	  case 3: classi->unknownPtR.laserC[classi->unknownPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->unknownPtR.numPuntos++;
		  break;
	  }
  }
/*fprintf(stderr,"staticPtR=%d\tmovingPtR=%d\tunknownPtR=%d\n",
							classi->staticPtR.numPuntos, 
							classi->movingPtR.numPuntos,
							classi->unknownPtR.numPuntos);  
*/
  // Keep the robot pose and the unknown points for the next iteration
  for (i=0; i<classi->unknownPtR.numPuntos; i++) {
	  unknown_k1.laserC[i]=classi->unknownPtR.laserC[i];
  }
  unknown_k1.numPuntos=classi->unknownPtR.numPuntos;
  Srobot_k1=*SRobot;


}







/**************************************************
 Points of laser are clasified into static, dynamic or unknown according to the map 
 and the tracked objects

void clasificarPuntos(float *laser, float **mapa, TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi)

 INPUT:
	laser
	mapa
	KFS	
	Srobot
	Smap
   
 OUTPUT:
	classi
   
***************************************************/
void clasificarPuntos2(float *laser, float mapa[][DIMMATRIZ], TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi) {

  // Variable definitions
  int I,J,i,started;			
  TnodoKFPT *sig;
  Tpf inPf;
  Tpcelda filterCells[maxNum];
  Tpcelda unknown_k1Cells[NUMPUNTOSLASER];
  static Tpcelda scanCells[NUMPUNTOSLASER];	  
    
  short assFilter, numFilter, outCnt;
  short i_s, j_s;
  short nsegments;
  Tpi segments[NUMPUNTOSLASER];
  Tsc PosMat,tmp2,tmp3;
  float dist,dist1,dist2;
  int ind1,ind2;
  float DIST_TH_SEG=0.2*0.2;

  // Compute the matrix coordinates of the previous unknown points  
  inversion_sis(SMap, &tmp2);  
  composicion_sis(&tmp2,&Srobot_k1,&PosMat);
  for (I=0; I<unknown_k1.numPuntos; I++) {
	  transfor_directa_p (unknown_k1.laserC[I].x,unknown_k1.laserC[I].y,
				&PosMat,&inPf);
	  SisM2celda(&inPf, &unknown_k1Cells[I]);
  }

  // Compute the matrix coordinates of the filters
  sig=KFS;
  numFilter=0;
  while (sig!=NULL){
	  inPf.x=sig->filter.x.data[0];
	  inPf.y=sig->filter.x.data[1];
	  transfor_inversa_p (sig->filter.x.data[0], sig->filter.x.data[1], SMap, &inPf);
	  SisM2celda(&inPf,&(filterCells[numFilter]));
	  numFilter=numFilter+1;
	  sig=(TnodoKFPT*)sig->next;
  }  

  // Loop over points to classify them
  // Compute the cartesian coordinates of the laser points
  // Compute the matrix coordinates of the scan
  // Use the map and the filters to label each point
  outCnt=0;
	
  inversion_sis(SMap, &tmp2);
  composicion_sis(&tmp2,SRobot,&tmp3);
  composicion_sis(&tmp3,&PosSensorRobot,&PosMat);

  classi->staticPtR.numPuntos=0; classi->movingPtR.numPuntos=0; classi->unknownPtR.numPuntos=0;
  for (i=0;i<NUMPUNTOSLASER;i++){
	  classi->tag[i]=0;
	  if (laser[i] <MAXRANGOLASER){
		    transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosSensorRobot,&classi->robotScan.laserC[i]);
			transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosMat,&inPf);			
			SisM2celda(&inPf,&scanCells[i]);	
			SisM2celda(&inPf,&classi->scanCells[i]);

			j_s=scanCells[i].col;
			i_s=scanCells[i].fil;
			if (!(i_s<0 || i_s>DIMMATRIZ-1 || j_s<0 || j_s>DIMMATRIZ-1)) {
				if (IsStaticProb_L1(i_s, j_s, mapa)) {			 
					classi->tag[i]=1;
				}
				else if (IsBeingTracked(i_s, j_s, filterCells, numFilter, &assFilter)) {
					classi->tag[i]=2;
				}
				else if (IsStaticProb_L2(i_s, j_s,mapa)) {
					classi->tag[i]=1;
				}
				else if (IsMovingProb(i_s, j_s,mapa)){
					classi->tag[i]=2;
				}
				else {
					for (J=0; J<unknown_k1.numPuntos; J++) {
						if (unknown_k1Cells[J].col==j_s && unknown_k1Cells[J].fil==i_s) {
							classi->tag[i]=1;
							break;
						}
					}
					if (J==unknown_k1.numPuntos) {				 
						classi->tag[i]=3;
					}
				}           
			}			
	  }
	  else {		  
		  transfor_directa_p (MAXRANGOLASER*cosAng[i],	MAXRANGOLASER*sinAng[i],
				&PosSensorRobot,&classi->outPtR.laserC[outCnt]);						
		  classi->robotScan.laserC[i]=classi->outPtR.laserC[outCnt];
		  classi->tag[i]=4;
		  outCnt++;			
	  }
  }
 
  classi->robotScan.numPuntos=NUMPUNTOSLASER;
  classi->outPtR.numPuntos=outCnt;    
  

  // Segmentate dynamic points 
  started=0;
  nsegments=0;
  if (classi->tag[0]==2){
	  started=1;
	  segments[nsegments].x=0;
  }

  for (i=1; i<NUMPUNTOSLASER-1;i++){
	  if (classi->tag[i]==2){
		  if (started){
			  dist=(classi->robotScan.laserC[i].x-classi->robotScan.laserC[i-1].x)*
				   (classi->robotScan.laserC[i].x-classi->robotScan.laserC[i-1].x) +
				   (classi->robotScan.laserC[i].y-classi->robotScan.laserC[i-1].y)*
				   (classi->robotScan.laserC[i].y-classi->robotScan.laserC[i-1].y);
			  if (dist>DIST_TH_SEG) {
				  segments[nsegments].y=i-1;				  				  
				  nsegments++;
				  segments[nsegments].x=i;
			  }			  
		  }
		  else{
			  started=1;			  
			  segments[nsegments].x=i;
		  }
	  }
	  else {
		  if (started){
			  segments[nsegments].y=i-1;
			  nsegments++;
			  started=0;
		  }
	  }
  }

  if (classi->tag[i]==2){
	  if (started){
		  dist=(classi->robotScan.laserC[i].x-classi->robotScan.laserC[i-1].x)*
			  (classi->robotScan.laserC[i].x-classi->robotScan.laserC[i-1].x) +
			  (classi->robotScan.laserC[i].y-classi->robotScan.laserC[i-1].y)*
			  (classi->robotScan.laserC[i].y-classi->robotScan.laserC[i-1].y);
		  if (dist>DIST_TH_SEG) 
			  segments[nsegments].y=i-1;				  				  			  			  		  
		  else
			  segments[nsegments].y=i;
	  }	  
  }
  else {
	  if (started){
		  segments[nsegments].y=i-1;
	  }
  }


  // Now filter out dynamic points using local minima
  for (i=0; i<nsegments;i++){
	  ind1=segments[i].x;
	  ind2=segments[i].y;
	  dist1=(classi->robotScan.laserC[ind1].x-classi->robotScan.laserC[ind1-1].x)*
		    (classi->robotScan.laserC[ind1].x-classi->robotScan.laserC[ind1-1].x) +
			(classi->robotScan.laserC[ind1].y-classi->robotScan.laserC[ind1-1].y)*
			(classi->robotScan.laserC[ind1].y-classi->robotScan.laserC[ind1-1].y);
	  dist2=(classi->robotScan.laserC[ind2].x-classi->robotScan.laserC[ind2+1].x)*
		    (classi->robotScan.laserC[ind2].x-classi->robotScan.laserC[ind2+1].x) +
			(classi->robotScan.laserC[ind2].y-classi->robotScan.laserC[ind2+1].y)*
			(classi->robotScan.laserC[ind2].y-classi->robotScan.laserC[ind2+1].y);
	  if ((ind1>0 && classi->tag[ind1-1]==1 && dist1<DIST_TH_SEG) ||
		  (ind2<NUMPUNTOSLASER && classi->tag[ind2+1]==1 && dist2<DIST_TH_SEG))	  
		  for (J=ind1; J<=ind2; J++)
			  classi->tag[J]=1;	  
  }

  // fill the output variables
  
  classi->staticPtR.numPuntos=0;
  classi->movingPtR.numPuntos=0;
  classi->unknownPtR.numPuntos=0;

  for (i=0; i<NUMPUNTOSLASER; i++){
	  switch(classi->tag[i]){
	  case 1: classi->staticPtR.laserC[classi->staticPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->staticPtR.numPuntos++;
		  break;
	  case 2: classi->movingPtR.laserC[classi->movingPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->movingPtR.numPuntos++;
		  break;
	  case 3: classi->unknownPtR.laserC[classi->unknownPtR.numPuntos]=classi->robotScan.laserC[i];
		  classi->unknownPtR.numPuntos++;
		  break;
	  }
  }
	  
  // Keep the robot pose and the unknown points for the next iteration
  for (I=0; I<classi->unknownPtR.numPuntos; I++) {
	  unknown_k1.laserC[I]=classi->unknownPtR.laserC[I];
  }
  unknown_k1.numPuntos=classi->unknownPtR.numPuntos;
  Srobot_k1=*SRobot;

}






/**************************************************
 Points of laser are clasified into static, dynamic or unknown according to the map 
 and the tracked objects

void clasificarPuntos(float *laser, float **mapa, TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi)

 INPUT:
	laser
	mapa
	KFS	
	Srobot
	Smap
   
 OUTPUT:
	classi
   
***************************************************/
/*void clasificarPuntosOld(float *laser, float mapa[][DIMMATRIZ], TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap,TclassPt *classi) {

  // Variable definitions
  int I,J,i;			
  TnodoKFPT *sig;
  Tpf inPf;
  Tpcelda filterCells[maxNum];
  Tpcelda unknown_k1Cells[NUMPUNTOSLASER];
  static Tpcelda scanCells[NUMPUNTOSLASER];	  
  short movingInd[NUMPUNTOSLASER];
    
  short assFilter, numFilter, cnt2, outCnt;
  short i_s, j_s;
  short nsegments;
  Tpi segments[maxNum];
  short I1;
  Tsc PosMat,tmp,tmp2,tmp3;

  // Compute the matrix coordinates of the previous unknown points  
  inversion_sis(SMap, &tmp2);  
  composicion_sis(&tmp2,&Srobot_k1,&PosMat);
  for (I=0; I<unknown_k1.numPuntos; I++) {
	  transfor_directa_p (unknown_k1.laserC[I].x,unknown_k1.laserC[I].y,
				&PosMat,&inPf);
	  SisM2celda(&inPf, &unknown_k1Cells[I]);
  }

  // Compute the matrix coordinates of the filters
  sig=KFS;
  numFilter=0;
  while (sig!=NULL){
	  inPf.x=sig->filter.x.data[0];
	  inPf.y=sig->filter.x.data[1];
	  transfor_inversa_p (sig->filter.x.data[0], sig->filter.x.data[1], SMap, &inPf);
	  SisM2celda(&inPf,&(filterCells[numFilter]));
	  numFilter=numFilter+1;
	  sig=sig->next;
  }  

  // Loop over points to classify them
  // Compute the cartesian coordinates of the laser points
  // Compute the matrix coordinates of the scan
  // Use the map and the filters to label each point
  cnt2 = 0; outCnt=0;
	
  inversion_sis(SMap, &tmp2);
  composicion_sis(&tmp2,SRobot,&tmp3);
  composicion_sis(&tmp3,&PosSensorRobot,&PosMat);

  classi->staticPtR.numPuntos=0; classi->movingPtR.numPuntos=0; classi->unknownPtR.numPuntos=0;
  for (i=0;i<NUMPUNTOSLASER;i++){
	  if (laser[i] <MAXRANGOLASER){
		    transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosSensorRobot,&classi->robotScan.laserC[cnt2]);
			transfor_directa_p (laser[i] * cosAng[i], laser[i] * sinAng[i],
				&PosMat,&inPf);			
			SisM2celda(&inPf,&scanCells[cnt2]);	
			SisM2celda(&inPf,&classi->scanCells[cnt2]);

			j_s=scanCells[cnt2].col;
			i_s=scanCells[cnt2].fil;
			if (!(i_s<0 || i_s>DIMMATRIZ-1 || j_s<0 || j_s>DIMMATRIZ-1)) {
				if (IsStaticProb_L1(i_s, j_s, mapa)) {			 
					classi->staticPtR.laserC[classi->staticPtR.numPuntos]=classi->robotScan.laserC[cnt2];
					classi->staticPtR.numPuntos++;			 
					classi->tag[i]=1;
				}
				else if (IsBeingTracked(i_s, j_s, filterCells, numFilter, &assFilter)) {
					classi->movingPtR.laserC[classi->movingPtR.numPuntos]=classi->robotScan.laserC[cnt2];
					movingInd[classi->movingPtR.numPuntos]=cnt2;
					classi->movingPtR.numPuntos++;		 			 
					classi->tag[i]=2;
				}
				else if (IsStaticProb_L2(i_s, j_s,mapa)) {
					classi->staticPtR.laserC[classi->staticPtR.numPuntos]=classi->robotScan.laserC[cnt2];
					classi->staticPtR.numPuntos++;
					classi->tag[i]=1;
				}
				else if (IsMovingProb(i_s, j_s,mapa)){
					classi->movingPtR.laserC[classi->movingPtR.numPuntos]=classi->robotScan.laserC[cnt2];
					movingInd[classi->movingPtR.numPuntos]=cnt2;
					classi->movingPtR.numPuntos++;			 
					classi->tag[i]=2;
				}
				else {
					for (J=0; J<unknown_k1.numPuntos; J++) {
						if (unknown_k1Cells[J].col==j_s && unknown_k1Cells[J].fil==i_s) {
							classi->staticPtR.laserC[classi->staticPtR.numPuntos]=classi->robotScan.laserC[cnt2];
							classi->staticPtR.numPuntos++;
							classi->tag[i]=1;
							break;
						}
					}
					if (J==unknown_k1.numPuntos) {
						classi->unknownPtR.laserC[classi->unknownPtR.numPuntos]=classi->robotScan.laserC[cnt2];
						classi->unknownPtR.numPuntos++;				 
						classi->tag[i]=3;
					}
				}           
			}
			cnt2++;
	  }
	  else {		  
		  transfor_directa_p (MAXRANGOLASER*cosAng[i],	MAXRANGOLASER*sinAng[i],
				&PosSensorRobot,&classi->outPtR.laserC[outCnt]);						
		  classi->tag[i]=4;
		  outCnt++;			
	  }
  }
 
  classi->robotScan.numPuntos=cnt2;
  classi->outPtR.numPuntos=outCnt;  

  // Keep the robot pose and the unknown points for the next iteration
  for (I=0; I<classi->unknownPtR.numPuntos; I++) {
	  unknown_k1.laserC[I]=classi->unknownPtR.laserC[I];
  }
  unknown_k1.numPuntos=classi->unknownPtR.numPuntos;
  Srobot_k1=*SRobot;


  // Now filter out dynamic spurius using local minima for moving objects
//  if (classi->movingCnt>0) {
    nsegments=0;
	segments[nsegments].x=classi->movingInd[0];    	        
    
    for (J=1;J<classi->movingPtR.numPuntos; J++) {
		I=movingInd[J];
		I1=movingInd[J-1];
		if ( abs(scanCells[I].col-scanCells[I1].col)>1 || 
			 abs(scanCells[I].fil-scanCells[I1].fil)>1) {        
            segments[nsegments].y=I1;             
            nsegments=nsegments+1;
            segments[nsegments].x=I;            
        }
    }
    
    segments[nsegments].y=classi->movingInd[classi->movingCnt-1];    


	classi->movingCnt=0;    
	classi->movingSpuriusCnt=0;
    for (I=0;I<nsegments;I++) {
        if (segments[I].x>0 && 
			(abs(scanCells[segments[I].x].col-scanCells[segments[I].x-1].col)<2 &&
			 abs(scanCells[segments[I].x].fil-scanCells[segments[I].x-1].fil)<2 &&
			 classi->tag[segments[I].x-1]==0) ||
			(segments[I].y<classi->robotScan.numPuntos && 
			abs(scanCells[segments[I].y].col-scanCells[segments[I].y+1].col)<2 &&
			abs(scanCells[segments[I].y].fil-scanCells[segments[I].y+1].fil)<2 &&
			classi->tag[segments[I].y+1]==0) ){
				
			for (J=segments[i].x;J<=segments[I].y;J++) {
				classi->movingInd[classi->movingCnt]=J;
				classi->movingCnt++;
			}               
		}
		else {
			classi->movingSpuriusPtR[classi->movingSpuriusPtR.numPuntos]=
		}
    }        
  }

}*/



