/***************************************************/
/* Last Revised:
$Id: IntMatND.c,v 1.2 2004/02/11 21:12:49 jminguez Exp jminguez $
*/
/***************************************************/

#include <stdio.h>
#include "IntMatND.h"
#include "Matriz.h"
#include "LinePl.h"
#include "calcul.h"
#include "TData.h"
#include <math.h>
#include "geometria.h"
#include "nd.h"


/**************** **************** ***************/
/**************** PROTOS FUNCIONES ***************/
/**************** **************** ***************/

static void celda2Matriz2(int fil, int col, Tpf *out);
static int ObtenerSectorP2(float p) ;

/**************** **************** ***************/
/**************** FUNCIONES EXTERN ***************/
/**************** **************** ***************/

void InitMatND (TParametrosND *NDparametros){

  printf("-- Iniciar el ND .");

  NDparametros->delante = 0.35; //0.85;//0.35;//0.85;
  NDparametros->detras = 0.32; //0.235;//0.32;//0.235;
  NDparametros->izquierda = 0.25; //0.335;//0.25;//0.335;

  NDparametros->dsmax= 0.40; //0.6;
  NDparametros->dsmin=NDparametros->dsmax/4.0F;
  NDparametros->engorde= 0.05; //0.1;
  
  NDparametros->vlmax=0.3;
  NDparametros->vamax=0.5;

  NDparametros->almax=0.75;
  NDparametros->aamax=0.75;

  NDparametros->discontinuidad = 0.55; //0.67;//0.55;//0.67;
  NDparametros->T=0.2;


  NDparametros->geometriaRect=1; // Si es cuadrado o no

  NDparametros->holonomo=0;

  NDparametros->laser=0.16F; //0.72F; // Compatibilidad josuna.

  printf(".");

  InicializarND(NDparametros);

  printf(" OK\n"); 

}


void ComputaMapa(float mapa[][DIMMATRIZ],
		 Tsc *sisMat, 
		 Tsc *sisrob,
		 TInfoEntorno *entorno,
		 Tscan *scan) {

  int i,j;
  Tsc sisOut,sisRobMat;
  Tpf sol,out;
  float max_valor;
  float dist,ang,d;
  int sec;
  Tpfp sectorizadoP[SECTORES_PL];


  max_valor = 2 * CUADRADO(DIMMATRIZ*DIMCELDA);

  inversion_sis(sisrob,&sisOut);
  composicion_sis(&sisOut,sisMat,&sisRobMat);

  for (i=0; i<SECTORES_PL; i++)
    sectorizadoP[i].r=max_valor;

  for (i=0; i<scan->numPuntos; i++) {
	  car2pol(&scan->laserC[i],&scan->laserP[i]);
	  sec=ObtenerSectorP2(scan->laserP[i].t);
	  if (sectorizadoP[sec].r>scan->laserP[i].r) {
		  sectorizadoP[sec].r=scan->laserP[i].r*scan->laserP[i].r;
		  sectorizadoP[sec].t=scan->laserP[i].t;
	  }
  }


  for (i=0; i<DIMMATRIZ; i++)
    for (j=0; j<DIMMATRIZ; j++)
      if (mapa[i][j]>0){
	celda2Matriz2(i,j,&out);
	transfor_directa_p(out.x,out.y,&sisRobMat,&sol);
	dist=CUADRADO(sol.x)+CUADRADO(sol.y);
	ang=ARCOTANGENTE(sol.x,sol.y);
	sec=ObtenerSectorP2(ang);
	
	if (sectorizadoP[sec].r>dist){
	  sectorizadoP[sec].r=dist;
	  sectorizadoP[sec].t=ang;
	}
      }

  entorno->longitud = 0;
  for (i=0; i<SECTORES_PL; i++)
    if (sectorizadoP[i].r!=max_valor){
      d=sqrt(sectorizadoP[i].r);
      transfor_directa_p(d*cos(sectorizadoP[i].t),
			 d*sin(sectorizadoP[i].t),
			 sisrob,&sol);
      entorno->punto[entorno->longitud].x=sol.x;
      entorno->punto[entorno->longitud].y=sol.y;
      entorno->longitud++;
    }
}



static void celda2Matriz2(int fil, int col, Tpf *out){

  out->x=(0.5F+col)*DIMCELDA;
  out->y=(DIMMATRIZ-0.5F-fil)*DIMCELDA;

}

static int ObtenerSectorP2(float p) {

  #define FACTOR (-SECTORES_PL/(2.0F*PI))
  #define SUMANDO ((SECTORES_PL+1.0F)/2.0F)

  return ((int)(FACTOR*p+SUMANDO))%SECTORES_PL;

  #undef SUMANDO
  #undef FACTOR
}

