/***************************************************/
/* Last Revised:
$Id: Matriz.c,v 1.2 2004/02/11 21:13:00 jminguez Exp jminguez $
*/
/***************************************************/

#include "Matriz.h"
#include "TData.h"
#include "calcul.h"
#include <stdlib.h>
#include <stdio.h>

/* ---------------------------------------------------------------------*/
/*  protos de las estaticas                                             */
/* ---------------------------------------------------------------------*/

static void MovArribaIzquierda(float a[][DIMMATRIZ],
			       int arriba,
			       int izquierda);
static void MovAbajoIzquierda(float a[][DIMMATRIZ],
			      int abajo,
			      int izquierda);
static void MovAbajoDerecha(float a[][DIMMATRIZ],
			    int abajo,
			    int derecha);
static void MovArribaDerecha(float a[][DIMMATRIZ],
			     int arriba,
			     int derecha);

/* ---------------------------------------------------------------------*/
/* ---------------------------------------------------------------------*/
/*  Funciones externas                                                  */
/* ---------------------------------------------------------------------*/
/* ---------------------------------------------------------------------*/

/* -------------------------------------------------
  Funcion de inicializacion de la matriz del entorno 
*/

void InitMatriz(float a[][DIMMATRIZ], Tsc *Smatriz, Tsc *Srobot){

  short i,j;

  printf("-- Initiating Matriz .");
  
  for (i=0;i<DIMMATRIZ;i++)
    for (j=0;j<DIMMATRIZ;j++)
      a[i][j]=DESCONOCIDO;    
  
  printf(".");
  
  Smatriz->x=Srobot->x-DIMMATRIZ*DIMCELDA/2.0f; 
  Smatriz->y=Srobot->y-DIMMATRIZ*DIMCELDA/2.0f;
  Smatriz->tita=0;

  printf(" OK!!\n");
  
}

/* -------------------------------------------------------------
  Funciones de transformacion. Sistema matrix -> celda
  El sistema matriz esta situado en la esquina inferior izquierda
  No se comprueba si esta dentro de la matriz, eso hazlo tu fuera
*/

void SisM2celda(Tpf *in,Tpcelda *out){

  if (in->y<0.0F)
    out->fil=(DIMMATRIZ-1)-(int)(in->y/DIMCELDA)+1;
  else
    out->fil=(DIMMATRIZ-1)-(int)(in->y/DIMCELDA);

  if (in->x<0.0F)
    out->col=-1-(int)(in->x/DIMCELDA);
  else
    out->col=(int)(in->x/DIMCELDA);
}

void Celda2M(Tpcelda *in,Tpf *out){

  out->x=(0.5F+in->col)*DIMCELDA;
  out->y=(DIMMATRIZ-0.5-in->fil)*DIMCELDA;

}


int PuntoMatInMatriz(Tpf *punto){

  Tpcelda out;
  
  SisM2celda(punto,&out);
  
  if (out.fil>=0 && out.fil<DIMMATRIZ &&
      out.col>=0 && out.col<DIMMATRIZ)
    return 1;
  else
    return 0;
}

int CeldaInMatriz(Tpcelda *in){

  if (in->fil>=0 && in->fil<DIMMATRIZ &&
      in->col>=0 && in->col<DIMMATRIZ)
    return 1;
  else
    return 0;

}

int RecomputaPuntoFinal(Tpf *pInit, Tpf *pFinal, Tpf *pNuevo){
  
  Tpf sol;
  
  if (corte_segmentos ( pInit->x, pInit->y, pFinal->x, pFinal->y,
			DIMCELDA/2, DIMCELDA/2, DIMMATRIZ*DIMCELDA-DIMCELDA/2, DIMCELDA/2,
			&sol)){
    *pNuevo=sol;
    return 1;
  }
      
      if (corte_segmentos ( pInit->x, pInit->y, pFinal->x, pFinal->y,
			    DIMMATRIZ*DIMCELDA-DIMCELDA/2, DIMCELDA/2,
			    DIMMATRIZ*DIMCELDA-DIMCELDA/2, DIMMATRIZ*DIMCELDA-DIMCELDA/2, 
			&sol)){
    *pNuevo=sol;
    return 1;
  }

  if (corte_segmentos ( pInit->x, pInit->y, pFinal->x, pFinal->y,
			DIMMATRIZ*DIMCELDA-DIMCELDA/2, DIMMATRIZ*DIMCELDA-DIMCELDA/2, 
			DIMCELDA/2, DIMMATRIZ*DIMCELDA-DIMCELDA/2,
			&sol)){
    *pNuevo=sol;
    return 1;
  }


  if (corte_segmentos ( pInit->x, pInit->y, pFinal->x, pFinal->y,
			DIMCELDA/2, DIMMATRIZ*DIMCELDA-DIMCELDA/2,
			DIMCELDA/2, DIMCELDA/2, 
			&sol)){
    *pNuevo=sol;
    return 1;
  }

      return 0;

}

/* -----------------------------------------------------------------
   Funciones de movimiento de la matriz
*/

void MovimientoMatriz(Tsc *Srobot, Tsc *Smatriz,
		      float a[][DIMMATRIZ]){

  Tpcelda CelR,CelC;
  Tpf point,p2;
  float centro;
  float dx,dy;
  int desCol,desFil;
  int izquierda = 0;
  int derecha = 0;
  int arriba = 0;
  int abajo = 0;

//   fprintf(stderr, "entro en mov matrix transf_inversa\n");
  /* Transformo el robot a la matriz */
  transfor_inversa_p(Srobot->x,Srobot->y,Smatriz,&point);
//   fprintf(stderr, "salgo de mov matrix transf_inversa\n");
  /* Compruebo si esta dentro del circulo */
  centro = DIMMATRIZ/2*DIMCELDA;
  dx=point.x-centro;
  dy=point.y-centro;
  if (dx*dx+dy*dy>RADIO_CENTRAL*RADIO_CENTRAL){
//     fprintf(stderr, "entro en mov matrix calculo movimiento\n");
    /* Si, calculo el movimiento */
    SisM2celda(&point,&CelR);
    p2.x=centro; p2.y=centro;
    SisM2celda(&p2,&CelC);
    desCol=CelR.col-CelC.col; desFil=CelR.fil-CelC.fil;
//     fprintf(stderr, "descol = %d, desfil = %d\n",desCol,desFil);
    if (desCol<0) izquierda=abs(desCol);
    else derecha=desCol;
    if (desFil<0) arriba=abs(desFil);
    else abajo=desFil;      
    
//     fprintf(stderr, "entro en mov matrix Desplazamiento\n");
    /* Desplazo la matriz interior comentado DANI*/
    if (arriba && derecha) 
      MovArribaDerecha( a , arriba , derecha );
    else if (abajo && derecha)
      MovAbajoDerecha( a, abajo, derecha );
    else if (abajo && izquierda)
      MovAbajoIzquierda(a, abajo , izquierda );
    else if (arriba && izquierda)
      MovArribaIzquierda(a,  arriba , izquierda );
    else if (arriba)
      MovArribaIzquierda(a,  arriba , 0 );
    else if (derecha)
      MovArribaDerecha(a,  0 , derecha );
    else if (izquierda)
      MovArribaIzquierda(a,  0 , izquierda );
    else if (abajo)
      MovAbajoIzquierda(a,  abajo , 0 );
//     fprintf(stderr, "salgo de mov matrix Desplazamiento\n");
    /* Actualizo las coordenadas de la matriz */
    Smatriz->x = Smatriz->x + desCol*DIMCELDA;
    Smatriz->y = Smatriz->y - desFil*DIMCELDA;    
  }
}

/******************************************************/
/**** funciones estaticas ****************************/
/******************************************************/

static void MovArribaDerecha(float a[][DIMMATRIZ],
		      int arriba,
		      int derecha){
//     fprintf(stderr, "Entro MovArribaDerecha, arriba %d, derecha %d\n",arriba,derecha);
  int i,j;
  /* Actualizo el centro de la matriz */
  for (i=DIMMATRIZ-1;i>=arriba;i--)
    for (j=0;j<=DIMMATRIZ-derecha-1;j++)
      a[i][j]=a[i-arriba][j+derecha];
  
  /* Relleno los lados de vacio */
  for ( i=0;i<DIMMATRIZ;i++)
    for ( j=0;j<arriba;j++)
      a[j][i]=DESCONOCIDO;
  for ( i=0;i<DIMMATRIZ;i++)
    for ( j=0;j<derecha;j++)
      a[i][DIMMATRIZ-1-j]=DESCONOCIDO;  
}
    
static void MovAbajoDerecha(float a[][DIMMATRIZ],
		      int abajo,
		      int derecha){
//     fprintf(stderr, "Entro MovAbajoDerecha\n");
  int i,j;
  /* Actualizo el centro de la matriz */
  for ( i=0;i<=DIMMATRIZ-abajo;i++)
    for (j=0;j<=DIMMATRIZ-derecha-1;j++)
      a[i][j]=a[i+abajo][j+derecha];
  
    /* Relleno los lados de vacio */
  for ( i=0;i<DIMMATRIZ;i++)
    for (j=0;j<abajo;j++)
      a[DIMMATRIZ-1-j][i]=DESCONOCIDO;
  for ( i=0;i<DIMMATRIZ;i++)
    for (j=0;j<derecha;j++)
      a[i][DIMMATRIZ-1-j]=DESCONOCIDO;  
}

static void MovAbajoIzquierda(float a[][DIMMATRIZ],
		       int abajo,
		       int izquierda){
//     fprintf(stderr, "Entro MovAbajoIzquierda\n");
  int i,j;
  
  /* Actualizo el centro de la matriz */
  for (i=0;i<=DIMMATRIZ-abajo;i++)
    for (j=DIMMATRIZ-1;j>=izquierda;j--)
      a[i][j]=a[abajo+i][j-izquierda];
  
  /* Relleno los lados de vacio */
  for ( i=0;i<DIMMATRIZ;i++)
    for (j=0;j<abajo;j++)
      a[DIMMATRIZ-1-j][i]=DESCONOCIDO;
  for (i=0;i<DIMMATRIZ;i++)
    for (j=0;j<izquierda;j++)
      a[i][j]=DESCONOCIDO;  
}

static void MovArribaIzquierda(float a[][DIMMATRIZ],
		       int arriba,
		       int izquierda){
//     fprintf(stderr, "Entro MovArribaIzquierda\n");
  int i,j;

  /* Actualizo el centro de la matriz */
  for (i=DIMMATRIZ-1;i>=arriba;i--)
    for (j=DIMMATRIZ-1;j>=izquierda;j--)
      a[i][j]=a[i-arriba][j-izquierda];
  
  /* Relleno los lados de vacio */
  for ( i=0;i<DIMMATRIZ;i++)
    for (j=0;j<arriba;j++)
      a[j][i]=DESCONOCIDO;
  for ( i=0;i<DIMMATRIZ;i++)
    for (j=0;j<izquierda;j++)
      a[i][j]=DESCONOCIDO;  
}

