/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'w m p _ d e b u g _ c'
 *
 *                                      
 *
 *  File 'wmp_debug.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
 
#include "core/include/wmp_debug.h"
#include <stdarg.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include <stdlib.h>
#include <assert.h>
static unsigned char dbg_level;
static enum Output output;
static int  exclusive;
static FILE *fp;
static int tx;

void initDebug(enum Output _output, unsigned char _dbg_level, int _exclusive){	
	output=_output;
	dbg_level=_dbg_level;
	exclusive= _exclusive;
	if (output==file){
		fp=fopen("rt-wmp.dbg","w+");
	}else if(output==net){
			struct sockaddr_in tx_addr;
			tx_addr.sin_family=PF_INET;
		    tx_addr.sin_port=htons(4007);
		    tx_addr.sin_addr.s_addr=inet_addr(status.DEBUG_ADDR);
			if ((tx=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) <0){
				wmprintf(BOTTOM,"Error creating debug socket.\n");
			}
			int val=1;
			setsockopt(tx,SOL_SOCKET,SO_BROADCAST,&val,sizeof(val));
			if(connect(tx, (struct sockaddr*)&tx_addr,sizeof(struct sockaddr_in))<0){
				wmprintf(2,"Connect error on Debug socket:%s.\n",inet_ntoa(tx_addr.sin_addr));
			}
	}
}

void closeDebug(){
	if (output==file){
		fclose(fp);
	}
}

void dbgprintf(unsigned char level,const char *format, ...){
  	if (dbg_level==0) return;
  	char s[512];
  	va_list arg;
  	if ((!exclusive && level<=dbg_level) || (exclusive && level==dbg_level)){
  		va_start(arg, format);
  		vsprintf(&s[0], format, arg);
  		va_end(arg); 
  		switch (output){
  			case tty: 
  				wmprintf(BOTTOM,s);
  				break;
  			case file:
  				fprintf(fp,s);
  				break;
  			case net:

  				break;
  			default:
  				assert(CANT_ARRIVE);
  		}
  				
  	}
  	
}  
 
void lqmToString(char **lqm, char * s){
	 int i,j;
	 sprintf(s,"%s","");
     for (i=0;i<status.N_NODES;i++){
        for (j=0;j<status.N_NODES;j++){
            char s1[4];
            int v=0;
            if(i!=j) v=lqm[i][j];
            if (v<10) sprintf(s1,"  %d",v);
            else if (v<100) sprintf(s1," %d",v);
            else  sprintf(s1,"%d",v);
            strcat(s,s1);
            strcat(s," ");                               
        }
      	strcat(s,"\n");
     }
}

