/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'Q u e u e _ c'
 *
 *                                      
 *
 *  File 'Queue.c' 	                    				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include "include/Queue.h"

void initQueue(Queue_t *q, int size,int sRule,int rRule, char * name){
	sprintf(q->name,"%s",name);
	q->tsmsg=(TimeStampedMsg *) MALLOC (sizeof(TimeStampedMsg)*(size+1));
	memset(q->tsmsg,0,sizeof(TimeStampedMsg)*(size+1));
	q->queueSize=size;		
	q->sRule=sRule;
	q->rRule=rRule;	
	SEM_INIT(&q->mutex,0,1);
	SEM_INIT(&q->pauseMutex,0,1);
	q->lastPopped.valid=0;
	q->queueIdx=0;
	WAIT(q->pauseMutex);
}

void freeQueue(Queue_t *q){
	FREE((char*) q->tsmsg);
}

void setReplaceRule(Queue_t *q,int rRule){
	q->rRule=rRule;
}

void setSortRule(Queue_t *q,int sRule){
	q->sRule=sRule;
}

void selectionSort (Queue_t *q, int size ) { 		
	int i,j;
	for ( i = 0; i < (size-1); i++) {
		int mi = i;
		for ( j = i+1; j < size; j++) {
			if (q->tsmsg[j].m.priority > q->tsmsg[mi].m.priority) {
				mi=j;
			} else if(q->tsmsg[j].m.priority==q->tsmsg[mi].m.priority){				
				if (q->sRule==FIFO){
					if (q->tsmsg[j].ts <q->tsmsg[mi].ts) { //<
						mi = j;
					}
				} else if (q->sRule==LIFO){
					if (q->tsmsg[j].ts >q->tsmsg[mi].ts) {
						mi = j;
					}
				}
			}
		}
		TimeStampedMsg p=q->tsmsg[i];
		q->tsmsg[i]=q->tsmsg[mi];
		q->tsmsg[mi]=p;
	} 
} 

int pushMsg(Queue_t *q, Msg * p){//from protocol to out
	pushMsgWithTS(q,p,0);
	return 0;
}


int pushMsgWithTS(Queue_t *q, Msg * m, unsigned long ts){//from protocol to out
	WAIT(q->mutex);
	if (q->queueIdx<q->queueSize) {
		q->tsmsg[q->queueIdx].m=*m;
		if (ts>0){
			q->tsmsg[q->queueIdx].ts=ts;
		}else{			
			q->tsmsg[q->queueIdx].ts=getActualTimeus();
		}
		q->queueIdx++;
		if (q->sRule!=UNSORTED) selectionSort(q,q->queueIdx);
		SIGNAL(q->mutex);
		SIGNAL(q->pauseMutex);
		return 1;/*success*/
	}else{
		if (q->rRule==REPLACE){
			q->tsmsg[q->queueIdx].m=*m;
			if (ts>0){
				q->tsmsg[q->queueIdx].ts=ts;
			}else{			
				q->tsmsg[q->queueIdx].ts=getActualTimeus();
			}
			selectionSort(q,q->queueIdx+1);
			SIGNAL(q->mutex);
			return 1; /*success*/
		}else{			
			SIGNAL(q->mutex);
			return 0; /*queue full*/
		}
	}
} 
void rescheduleLast(Queue_t *q){
	if (q->lastPopped.valid)
		pushMsgWithTS(q,&(q->lastPopped.m),q->lastPopped.ts);
} 

void getLast(Queue_t *q, Msg* p){
	if (q->lastPopped.valid)
		*p=q->lastPopped.m;
	else
		popMsg(q,p);		
} 


long popMsg(Queue_t *q,Msg* p){ /* Pop the highest priority message*/
	WAIT(q->pauseMutex);
	WAIT(q->mutex);
	*p=q->tsmsg[0].m;
	long age=(long) (getActualTimeus()-q->tsmsg[0].ts);
	q->lastPopped.m=*p;
	q->lastPopped.valid=1;
	q->tsmsg[0].m.priority=-1;	
	selectionSort(q,q->queueIdx);
	q->queueIdx--;
	SIGNAL(q->mutex);	
	return age;
}




char getHighestPriority(Queue_t *q){
	WAIT(q->mutex);
	if (q->queueIdx==0 || q->sRule==UNSORTED){
		SIGNAL(q->mutex);
		 return -1; /*queue empty or unsorted vector*/
	}

	else {
		SIGNAL(q->mutex);
		return q->tsmsg[0].m.priority;
	}
}

unsigned long getNumOfElemInQueue(Queue_t *q){
	WAIT(q->mutex);
	unsigned int a=q->queueIdx;
	SIGNAL(q->mutex);
	return a;
}
unsigned long getFreePositions(Queue_t *q){
	WAIT(q->mutex);
	unsigned int a=(q->queueSize-q->queueIdx);
	SIGNAL(q->mutex);
	return a;
}

int getHighestPrioritysAge(Queue_t *q){
	WAIT(q->mutex);
	if (q->queueIdx==0 || q->sRule==UNSORTED){
		SIGNAL(q->mutex);
		return -1; /*queue empty or unsorted vector*/
	} else {	
		unsigned long long a=(getActualTimeus()-q->tsmsg[0].ts);
		a=a/1000; //to ms
		SIGNAL(q->mutex);
		return (int) a;
	}		 	
}

