/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'R T - W M P _ c'
 *
 *                                      
 *
 *  File 'RT-WMP.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include "config/compiler.h"
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <pthread.h>
#include "core/include/queues.h"
#include "include/rssi_average.h"
#include "include/wmp_com.h"
#include "include/manage.h"
#include "include/wmp_misc.h"
#include "include/wmp_debug.h"
#include "interface/wmp_interface.h"
#include "interface/wmp_protected_interface.h"
#include "include/nstat.h"
#include "include/lqm.h"
#include "extensions/extensions.h"    
//#include <assert.h>
  

Status status;  

void wmpRun(char node_id, char num_nodes, int GUI){
	wmprintf(TTY,"RT-WMP Node Starting...\n",node_id);
	wmpFrame * rxb;
	wmpFrame * p;
	p=(wmpFrame *) &(status.buffer[0]);
	rxb=(wmpFrame *) &(status.rxBuff[0]);
	wmpReadConfiguration(&status);
	if (node_id>=0){ 
		status.id=node_id;
	} 
	if ( num_nodes>0){
		status.N_NODES=num_nodes;
	}
	if (GUI>=0){
		status.GUI=GUI;
	}
	init_lqm(status.N_NODES);
	fill_lqm(100);
	initDijkstra(status.N_NODES);
	initQueues();
	initMobileAverage();
	initDebug((enum Output)status.DEBUG_OUTPUT, status.DEBUG_LEVEL,status.DEBUG_TYPE);		
	init_nstat(status.N_NODES);	
	if (!initLowLevelCom()){
		wmprintf(TTY,"Low Level Communication Layer Initialization error.\nHit any key to exit.\n");
		getchar();
		exit(1);
	}
	INIT_EXTENSIONS();
	wmpPluginStart();
	sleep(1);
	initGUI(status.GUI,status.N_NODES);	
	
	/*WORKING CODE*/
	wmprintf(TTY,"Node %d of %d is up and Running...\n",status.id,status.N_NODES);
	if (status.quiet) wmprintf(TTY,"\n*** WARNING -- QUIET MODE *** \n\n",status.id,status.N_NODES);
	int rtnCode=RECEIVE;	
	status.timeStart=getActualTimeus();
	while (1){
		status.alive=1;
		int RXUCQ=getNumOfElementInRXUCQueue();
		int TXUCQ=getNumOfElementInTXUCQueue();
	 	
		char yy[2000];
		//lqmToString(lqm_get_ptr(),yy);
		//wmprintf(LQM,yy);
		//wmprintf(MIDRIGHT,"S:%d Status:%d\n", p->hdr.serial,rtnCode);
		//wmprintf(BOTTOM,"Enter in %d\n",rtnCode);
		switch (rtnCode){
			case RECEIVE:
				rtnCode=wmpReceive(p);
				break;
			case EVALUATE_TOKEN:				
				rtnCode=evaluate_token(p);
				break;
			case EVALUATE_AUTHORIZATION:				
				rtnCode=evaluate_authorization(p);
				break;
			case EVALUATE_MESSAGE:				
				rtnCode=evaluate_message(p);
				break;
			case EVALUATE_DROP_TOKEN:				
				rtnCode=RECEIVE;
				break;
			case EVALUATE_FOREIGN:
				rtnCode=RECEIVE;				
				break;
			case WAIT_ACK:				
				rtnCode=wmpWaitAck(p);				
				break;
			case NEW_TOKEN:
				rtnCode=createNewToken(p);
				break; 
			case TO_TOKEN_EXPIRED:
				rtnCode=manage_token_expired_timeout(p);
				break;
			case TO_AUTHORIZATION_EXPIRED:
				rtnCode=manage_authorization_expired_timeout(p);				
				break;
			case TO_MESSAGE_EXPIRED:
				rtnCode=manage_message_expired_timeout(p);				
				break;  
			case SEND:
				status.highestSerial++;
				p->hdr.serial=status.highestSerial;
				memcpy((char*) status.sendBak,p,MTU);
				status.waitRemaining=status.TIMEOUT;				
				status.retries=0;
				rtnCode=wmpSend(p);
				break;
			case RETRY:
				status.waitRemaining=status.TIMEOUT;				
				rtnCode=wmpSend((wmpFrame*)status.sendBak);
				break;
			case NET_INACTIVITY:
				status.highestSerial=0;
				rtnCode=NEW_TOKEN;
				break;
			case ABORT:
				wmpExit();
				break;
			default:
				wmprintf(BOTTOM,"Unmanaged State (%d) at main loop. Application will exit\n",rtnCode);
				getchar();
				wmpExit();
		}
	}
}

int wmpExit(){
	fprintf(stderr,"h0\n");
	closeLowLevelCom();
	fprintf(stderr,"h1\n");
	free_lqm();
	fprintf(stderr,"h2\n");
	free_nstat();	
	fprintf(stderr,"h3\n");
	freeQueues();
	fprintf(stderr,"h4\n");
	freeMobileAverage();
	fprintf(stderr,"h5\n");
	CLOSE_EXTENSIONS();
	fprintf(stderr,"h6\n");
	exitGUI();
	fprintf(stderr,"h7\n");
	closeDebug();
	fprintf(stderr,"h8\n");
	wmprintf(TTY,"\n*** RT-WMP NODE %d TERMINATED.\n",status.id);
	return 0;
}

