/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'q u e u e s_ c'
 *
 *                                      
 *
 *  File 'queues.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/

#include "include/queues.h"

Queue_t rxUCQueue;
Queue_t txUCQueue;


void initQueues(){	
	initQueue(&rxUCQueue,50,FIFO,REPLACE, "RX");	
	initQueue(&txUCQueue,50,FIFO,REPLACE, "TX");
}

void freeQueues(){	
	freeQueue(&rxUCQueue);	
	freeQueue(&txUCQueue);
}

void pushRcvdUCMsg(Msg * p){
	//fprintf(stderr,"Received UC Msg : %s\n",p->data);
	pushMsg(&rxUCQueue,p);
}

long popUCMsgToSend(Msg * p){
	return popMsg(&txUCQueue,p);
}

/* GETNUMELEMENTS FUNCTIONS*/

int getNumOfElementInRXUCQueue(){
		return getNumOfElemInQueue(&rxUCQueue);
}


int getNumOfFreePositionsInTXUCQueue(){
	return getFreePositions(&txUCQueue);
}


int getNumOfElementInTXUCQueue(){
	return getNumOfElemInQueue(&txUCQueue);
}
	
int getHighestPriorityInTxUCQueue(){
	return getHighestPriority(&txUCQueue);
}

int getHighestPriorityAgeInTxUCQueue(){
	return getHighestPrioritysAge(&txUCQueue);
}

/* USER */

int wmpGetNumOfFreePositionsInTXUCQueue(){
	return getNumOfFreePositionsInTXUCQueue();
}

int wmpGetNumOfElementInTXUCQueue(){ //EXPORTED FUNCTION
	return  getNumOfElementInTXUCQueue();
}



int wmpGetNumOfElementInRXUCQueue(){ //EXPORTED FUNCTION
	return  getNumOfElementInRXUCQueue();
}


/* PUSH & POP FUNCTIONS*/

int wmpPushUCMsg(Msg *p){
	pushMsg(&txUCQueue,p);
	return 1;
}

int wmpPopUCMsg(Msg *p){
	popMsg(&rxUCQueue,p);
	return 1;	
}


void wmpUCPush(char * data, unsigned short len, char dest, char priority){
	Msg m;
	memcpy(&m.data[0],data,len);
	m.len=len;
	m.priority=priority;
	m.dest=dest;
	//m.protocol=0;
	//m.offset=0;
	wmpPushUCMsg(&m);	
}

void wmpUCPop(char nqueue, char * data, unsigned short* len, char* source, char* priority, char* protocol){
	Msg m;
	wmpPopUCMsg(&m);
	memcpy(data,&m.data[0],m.len);
	if (len!=NULL){
		*len=m.len;	
	}
	if (source!=NULL){
		*source=m.src;
	}
	if (priority!=NULL){
		*priority=m.priority;
	}
	if (protocol!=NULL){
		*priority=m.priority;
	}
}
