/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                  		 'w m p _ c o m _ c'
 *
 *                                      
 *
 *  File 'wmp_com.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include "include/wmp_com.h"

int timer_initied=0;

void updateLocalData(wmpFrame *p){
	int i,j;
	/* Calculate RSSI */
	float f_rssi=(float) p->hdr.rssi;
	f_rssi=100.0*f_rssi/(status.max_rssi*1.0);
	if (f_rssi>99) f_rssi=99;
	p->hdr.rssi=(char) f_rssi;
	rssi_new_frame(p->hdr.from,p->hdr.rssi);	
	status.retries=0;
	
	/*if is a TOKEN update local LQM */	
	if (p->hdr.type == TOKEN){               
		char *cp= (char *) p;
		cp+=sizeof(Token_Hdr)+sizeof(Token); 
		for (i=0;i<status.N_NODES;i++){
			for (j=0;j<status.N_NODES;j++){
				if (i!=j) lqm_set_val(i,j,*cp);
				else nstat_set_byte(i,*cp);			
				cp++;
			}
		}		
	}
	
	/*  Actualize LocalLQM and control the HOLD_TIME */
	for (i=0;i<status.N_NODES;i++){
		if (i==status.id) continue;
		if (rssi_get_age(i)> status.hold_time){
			rssi_reset(i);
		}			
		if (!status.quiet){
			if (status.rssi_capable){
				lqm_set_val(status.id,i,rssi_get_averaged_rssi(i));
			}else{
				lqm_set_val(status.id,i,lqm_get_val(i,status.id));
				//status.localLQM[status.id][i]=status.localLQM[i][status.id];
			}
		}
	}	
}
 

void old_token(wmpFrame* t){
	if (status.quiet){
		return;
	}
	wmpFrame d;
	d.hdr.from=status.id;
	d.hdr.to=t->hdr.from;
	d.hdr.serial=t->hdr.serial; 
	d.hdr.type=DROP_TOKEN;
	//wmprintf(BOTTOM,"*** Sent DROP_TOKEN to Node %d (Serial:%d)\n",d.hdr.to,d.hdr.serial);
	status.drops++;
	llsend((char*) &d,sizeof(Token_Hdr));
}

/* Modified: added timeout to receive. If timeout (1 second or more depending on node id) expires, node creates
 * a new token with an higher serial to restart the network.
 * Net Restarting is prioritized on node id */ 

int wmpReceive(wmpFrame* p){	
	int net_inactivity_timeout;
	if (!status.quiet) net_inactivity_timeout=500+status.id*500;
	else {
		net_inactivity_timeout=(1<<24);
	}
	
	int rtn=llreceive((char*) &status.rxBuff[0], net_inactivity_timeout);
	if (rtn==EXPIRED){
		status.highestSerial+=10*(status.N_NODES-status.id);
		return NEW_TOKEN;
	}
	else if (rtn==RECEIVE_OK){
		wmpFrame * t= (wmpFrame*) &status.rxBuff[0];
		updateLocalData(t);
		int isForMe=t->hdr.to==status.id;
		int isNew=t->hdr.serial>status.highestSerial;
		int isDrop=t->hdr.type==DROP_TOKEN; 
		if (!isForMe) {
			memcpy(p,t,MTU);
			//if (status.ENHANCED_DROP) status.highestSerial=t->hdr.serial;					
			return EVALUATE_FOREIGN;
		}		
		if (isForMe && !isDrop){
			if (isNew){
				memcpy(p,t,MTU);
				status.highestSerial=t->hdr.serial;
				return (EVALUATE + t->hdr.type);
			} else {
				old_token(t);
				return RECEIVE;
			}
		}
		if (isForMe && isDrop) return RECEIVE;
	}
	ASSERT(CANT_ARRIVE);
}

int wmpWaitAck(wmpFrame* p){	
	int timeout=status.waitRemaining;		
	int rtn=UNDEF;
	if (timeout>0){
		long before=getActualTime();
		rtn=llreceive((char*) &(status.rxBuff[0]) , timeout);			
		long after=getActualTime();
		status.waitRemaining=timeout-(after-before);
	} else rtn=EXPIRED;
	
	if (rtn==RECEIVE_OK){ /* something received*/				
		wmpFrame * t= (wmpFrame*) &status.rxBuff[0];
		updateLocalData(t);	
		int isForMe=t->hdr.to==status.id;
		int isNew=t->hdr.serial>status.highestSerial;
		int isOld=t->hdr.serial<status.highestSerial;
		int isEqual=t->hdr.serial==status.highestSerial;
		int isDrop=t->hdr.type==DROP_TOKEN;
		
		if (!isForMe &&	isEqual &&	isDrop) 	return WAIT_ACK;			
		if (!isForMe && 1		&& 	isDrop) 	return WAIT_ACK;		
		if (!isForMe && !isNew 	&&  !isDrop) 	return WAIT_ACK;
		if (isForMe	&& 	isOld 	&&	isDrop) 	return WAIT_ACK;			
		if (isForMe && 	isEqual && 	isDrop) 	return RECEIVE;
		if (isForMe && 	isNew 	&& !isDrop){ 				 
			 /* ACK FRAME -> EVALUATE */
			 memcpy(p,t,MTU);
			 status.highestSerial=t->hdr.serial;		 
			 return (EVALUATE + t->hdr.type);			
		}
		if (!isForMe && isNew && !isDrop) {
			memcpy(p,t,MTU);
			status.highestSerial=t->hdr.serial;
			return EVALUATE_FOREIGN; /* ACK FRAME -> RECEIVE */
		}			
		if (isForMe && !isDrop && !isNew) {
			old_token(t);
			return WAIT_ACK;
		}
		ASSERT(CANT_ARRIVE);	
	} else if (rtn==EXPIRED){
		return (TO_EXPIRED + status.lastSentType);
	}
	ASSERT(CANT_ARRIVE);
}
void constructTokenWithLocalData(wmpFrame * t){
	int i,j;
	/* Create frame, setting reached and lqm */
	char *cp=(char*)t;
	cp+=sizeof(Token_Hdr)+sizeof(Token);
	for (i=0;i<status.N_NODES;i++){
		for (j=0;j<status.N_NODES;j++){
			if (i!=j)*cp=lqm_get_val(i,j);
			else *cp=nstat_get_byte(i);
			cp++;
		}
	}
}

int wmpSend(wmpFrame* t){	
	if (status.quiet) {		
		return RECEIVE;
	}
	int size=UNDEF;
	switch (t->hdr.type){
		case TOKEN:					
			size=sizeof(Token)+/*status.N_NODES+*/status.N_NODES*status.N_NODES+t->hdr.bc_len;
			constructTokenWithLocalData(t);
			break;
		case AUTHORIZATION:
			size=sizeof(Authorization)+t->hdr.bc_len;
			break;
		case MESSAGE:
			size=sizeof(Message)+t->msg.len+t->hdr.bc_len;
			status.sentMsg++;	
			break;
		case DISCOVER:
			size=0;
			break;
		default:
			ASSERT(CANT_ARRIVE);
	}
	t->hdr.from=status.id;	
	t->hdr.rssi=80;
	llsend((char*) t,sizeof(Token_Hdr)+size);
	status.lastSentType=t->hdr.type;
	status.sents++;
	return WAIT_ACK;
} 


 
